// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_MOJOM_PAGE_DISPLAY_CUTOUT_MOJOM_SHARED_H_
#define THIRD_PARTY_BLINK_PUBLIC_MOJOM_PAGE_DISPLAY_CUTOUT_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "third_party/blink/public/mojom/page/display_cutout.mojom-shared-internal.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace blink {
namespace mojom {
class DisplayCutoutSafeAreaDataView;



}  // namespace mojom
}  // namespace blink

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::blink::mojom::DisplayCutoutSafeAreaDataView> {
  using Data = ::blink::mojom::internal::DisplayCutoutSafeArea_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace blink {
namespace mojom {

enum class ViewportFit : int32_t {
  kAuto,
  kContain,
  kCover,
  kCoverForcedByUserAgent,
  kMinValue = 0,
  kMaxValue = 3,
};

inline std::ostream& operator<<(std::ostream& os, ViewportFit value) {
  switch(value) {
    case ViewportFit::kAuto:
      return os << "ViewportFit::kAuto";
    case ViewportFit::kContain:
      return os << "ViewportFit::kContain";
    case ViewportFit::kCover:
      return os << "ViewportFit::kCover";
    case ViewportFit::kCoverForcedByUserAgent:
      return os << "ViewportFit::kCoverForcedByUserAgent";
    default:
      return os << "Unknown ViewportFit value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(ViewportFit value) {
  return internal::ViewportFit_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class DisplayCutoutHostInterfaceBase {};

using DisplayCutoutHostPtrDataView =
    mojo::InterfacePtrDataView<DisplayCutoutHostInterfaceBase>;
using DisplayCutoutHostRequestDataView =
    mojo::InterfaceRequestDataView<DisplayCutoutHostInterfaceBase>;
using DisplayCutoutHostAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<DisplayCutoutHostInterfaceBase>;
using DisplayCutoutHostAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<DisplayCutoutHostInterfaceBase>;
class DisplayCutoutClientInterfaceBase {};

using DisplayCutoutClientPtrDataView =
    mojo::InterfacePtrDataView<DisplayCutoutClientInterfaceBase>;
using DisplayCutoutClientRequestDataView =
    mojo::InterfaceRequestDataView<DisplayCutoutClientInterfaceBase>;
using DisplayCutoutClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<DisplayCutoutClientInterfaceBase>;
using DisplayCutoutClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<DisplayCutoutClientInterfaceBase>;
class DisplayCutoutSafeAreaDataView {
 public:
  DisplayCutoutSafeAreaDataView() {}

  DisplayCutoutSafeAreaDataView(
      internal::DisplayCutoutSafeArea_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t top() const {
    return data_->top;
  }
  int32_t left() const {
    return data_->left;
  }
  int32_t bottom() const {
    return data_->bottom;
  }
  int32_t right() const {
    return data_->right;
  }
 private:
  internal::DisplayCutoutSafeArea_Data* data_ = nullptr;
};

class DisplayCutoutHost_NotifyViewportFitChanged_ParamsDataView {
 public:
  DisplayCutoutHost_NotifyViewportFitChanged_ParamsDataView() {}

  DisplayCutoutHost_NotifyViewportFitChanged_ParamsDataView(
      internal::DisplayCutoutHost_NotifyViewportFitChanged_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadValue(UserType* output) const {
    auto data_value = data_->value;
    return mojo::internal::Deserialize<::blink::mojom::ViewportFit>(
        data_value, output);
  }

  ViewportFit value() const {
    return static_cast<ViewportFit>(data_->value);
  }
 private:
  internal::DisplayCutoutHost_NotifyViewportFitChanged_Params_Data* data_ = nullptr;
};

class DisplayCutoutClient_SetSafeArea_ParamsDataView {
 public:
  DisplayCutoutClient_SetSafeArea_ParamsDataView() {}

  DisplayCutoutClient_SetSafeArea_ParamsDataView(
      internal::DisplayCutoutClient_SetSafeArea_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSafeAreaDataView(
      DisplayCutoutSafeAreaDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSafeArea(UserType* output) {
    auto* pointer = data_->safe_area.Get();
    return mojo::internal::Deserialize<::blink::mojom::DisplayCutoutSafeAreaDataView>(
        pointer, output, context_);
  }
 private:
  internal::DisplayCutoutClient_SetSafeArea_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace blink

namespace std {

template <>
struct hash<::blink::mojom::ViewportFit>
    : public mojo::internal::EnumHashImpl<::blink::mojom::ViewportFit> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::blink::mojom::ViewportFit, ::blink::mojom::ViewportFit> {
  static ::blink::mojom::ViewportFit ToMojom(::blink::mojom::ViewportFit input) { return input; }
  static bool FromMojom(::blink::mojom::ViewportFit input, ::blink::mojom::ViewportFit* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::ViewportFit, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::blink::mojom::ViewportFit, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::blink::mojom::ViewportFit>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::DisplayCutoutSafeAreaDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::DisplayCutoutSafeAreaDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::DisplayCutoutSafeArea_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->top = CallWithContext(Traits::top, input, custom_context);
    (*output)->left = CallWithContext(Traits::left, input, custom_context);
    (*output)->bottom = CallWithContext(Traits::bottom, input, custom_context);
    (*output)->right = CallWithContext(Traits::right, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::DisplayCutoutSafeArea_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::DisplayCutoutSafeAreaDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace blink {
namespace mojom {





inline void DisplayCutoutClient_SetSafeArea_ParamsDataView::GetSafeAreaDataView(
    DisplayCutoutSafeAreaDataView* output) {
  auto pointer = data_->safe_area.Get();
  *output = DisplayCutoutSafeAreaDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_PUBLIC_MOJOM_PAGE_DISPLAY_CUTOUT_MOJOM_SHARED_H_
